<?php

namespace Drupal\rng\Form\Entity;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\rng\Form\RegistrantFields;
use Drupal\rng\RegistrantsElementUtility;

/**
 * Form controller for registrants.
 */
class RegistrantForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $registrant = $this->entity;
    /** @var \Drupal\rng\Form\RegistrantFields $helper */
    $helper = new RegistrantFields($form, $form_state, $registrant);

    $form = $helper->getFields($form, $form_state, $registrant);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    RegistrantFields::validateForm($form, $form_state);
    return parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function copyFormValuesToEntity(EntityInterface $entity, array $form, FormStateInterface $form_state) {
    // @todo Change the autogenerated stub
    parent::copyFormValuesToEntity($entity, $form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $registrant = $this->entity;
    $is_new = $registrant->isNew();

    $registrant->save();

    if ($is_new) {
      $this->messenger()->addMessage($this->t('Registrant created.'));
    }
    else {
      $this->messenger()->addMessage($this->t('Registrant updated.'));
    }
  }

  /**
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array|null
   */
  public static function ajaxRegistrantType(array $form, FormStateInterface $form_state) {
    return RegistrantsElementUtility::findElement($form, $form_state);
  }

}
